<?php
require_once 'includes/Imengine/Imengine.php';
require_once 'includes/Imengine/adapters/IM_Adapter.php';
require_once 'includes/Imengine/adapters/GD_Adapter.php';
require_once 'includes/Imengine/adapters/S3_Adapter.php';
require_once 'includes/Imengine/adapters/Filestorage_Adapter.php';
require_once 'includes/Imengine/adapters/Voidstorage_Adapter.php';
require_once 'includes/oc_imengine.php';

/*
 * Takes uuid, type and source and returns an image
 */
if (isset($_GET['uuid'], $_GET['type']) || isset($_GET['imgsrc'], $_GET['type']) || isset($_GET['path'])) {
    define('WP_USE_THEMES', false);
    require('../../../wp-blog-header.php');

    $imgDrawer = new ImEngineDrawer();
    $imgDrawer->addMetaInformation($_GET);

    $ocImengine = new OC_Imengine($imgDrawer->q);
    $imageUrl = $imgDrawer->getGeneratedUrlString();

    if($imgDrawer->usesExternalImageSource()) {
        $imengine = $ocImengine->external_image($imgDrawer->getExternalImageUrl(), $imgDrawer->format);
    } else {
        $imengine = $ocImengine->image($imgDrawer->uuid, $imageUrl, $imgDrawer->format);
    }
    $func = (string)str_replace('+', '_', $imgDrawer->f);
    if (method_exists($imengine, $func)) {
        if ($imgDrawer->percent !== null) {
            $percent = preg_split("/\,/i", $imgDrawer->percent);
            $imengine->$func($imgDrawer->w, $imgDrawer->h, $percent);
        } elseif (($imgDrawer->crop_w > 0) && ($imgDrawer->crop_h > 0)) {
            $imengine->$func($imgDrawer->crop_w, $imgDrawer->crop_h, $imgDrawer->x, $imgDrawer->y, $imgDrawer->w, $imgDrawer->h, $imgDrawer->z);
        } elseif ($imgDrawer->has('zoom')) {
            $imengine->$func($imgDrawer->w, $imgDrawer->h, $imgDrawer->x, $imgDrawer->y, $imgDrawer->z);
        } elseif ($imgDrawer->x > 0 || $imgDrawer->y > 0) {
            $imengine->$func($imgDrawer->w, $imgDrawer->h, $imgDrawer->x, $imgDrawer->y);
        } else {
            $imengine->$func($imgDrawer->w, $imgDrawer->h);
        }
    } else {
        $imengine->cover(500, 'auto');
    }
}

class ImEngineDrawer
{
    protected $imgUrl;
    protected $meta = [];
    protected $typeOfRequest;

    const TYPE_V1 = 'v1';
    const TYPE_V2 = 'v2';

    public function __construct()
    {
        $this->setTypeOfRequest();
    }

    public function __set($key, $value)
    {
        if (!array_key_exists($key, $this->meta['meta'])) {
            $this->meta[$key] = $value;
        }
        return $this;
    }

    public function __get($key)
    {
        $function = "getAttribute".ucfirst($key);
        return $this->$function();
    }


    /**
     * Sets all the request parameters
     * Supports both old ? style parameters and new /article/img/ style
     * Add the information to the meta property
     * @param array $get
     */
    public function addMetaInformation(array $get)
    {
        if ($this->typeOfRequest == self::TYPE_V1) {
            $this->meta = $this->addMetaFromParameters($get);
        } elseif ($this->typeOfRequest == self::TYPE_V2) {
            $this->meta = $this->addMetaFromString($get);
        }
    }

    /**
     * Return array of meta information including image and path keys
     * @return array
     */
    public function getMetaInformation()
    {
        return $this->meta;
    }

    /**
     * Set typeOfRequest property to maintain backwards compatibility
     * Checks which kind of GET params that are available and decide which version
     * of image renderer we are using.
     */
    public function setTypeOfRequest()
    {
        if (isset($_GET['uuid'])) {
            $this->typeOfRequest = self::TYPE_V1;
        } elseif (isset($_GET['image'])) {
            $this->typeOfRequest = self::TYPE_V2;
        }
    }


    /**
     * Returns which type of request
     * @return mixed
     */
    public function getTypeOfRequest()
    {
        return $this->typeOfRequest;
    }


    /**
     * All meta data is passed at string with the _ delimeter.
     * Function splits up parts and returns and array with path, image and all available meta data
     * @param $imageMetaInformation
     * @return array
     */
    public function addMetaFromString($imageMetaInformation)
    {
        $parts = preg_split('/\_|\=/', $imageMetaInformation['meta']);
        $url = [];
        $url['image'] = $imageMetaInformation['image'];
        $url['path'] = $imageMetaInformation['path'];
        $c = 0;
        foreach ($parts as $key => $part) {
            if ($c % 2 == 0)
                $url['meta'][$part] = $parts[$c + 1];
            $c++;
        }

        return $url;
    }


    /**
     * Return array of all parameters set by query string parameters
     * @param $get
     * @return array
     */
    public function addMetaFromParameters($get)
    {
        $metaInformation = [];
        $metaInformation['meta']['uuid'] = isset($get['uuid']) ? $get['uuid'] : '';
        $metaInformation['meta']['type'] = isset($_GET['type']) ? $_GET['type'] : 'preview';
        $metaInformation['meta']['source_bool'] = (isset($get['source']) && $get['source'] == 'true') ? true : false;
        $metaInformation['meta']['f'] = isset($_GET['function']) ? $_GET['function'] : 'cover';
        $metaInformation['meta']['w'] = isset($_GET['width']) ? intval($_GET['width']) : 500;
        $metaInformation['meta']['h'] = isset($_GET['height']) ? intval($_GET['height']) : 'auto';
        $metaInformation['meta']['x'] = isset($_GET['x']) ? intval($_GET['x']) : 0;
        $metaInformation['meta']['y'] = isset($_GET['y']) ? intval($_GET['y']) : 0;
        $metaInformation['meta']['crop_w'] = isset($_GET['crop_w']) ? intval($_GET['crop_w']) : 0;
        $metaInformation['meta']['crop_h'] = isset($_GET['crop_h']) ? intval($_GET['crop_h']) : 0;
        $metaInformation['meta']['z'] = isset($_GET['z']) ? $_GET['z'] : null;
        $metaInformation['meta']['q'] = isset($_GET['q']) ? $_GET['q'] : 60;
        $metaInformation['meta']['percent'] = isset($_GET['percent']) ? $_GET['percent'] : null;
        $metaInformation['meta']['externalImgSrc'] = isset($_GET['imgsrc']) ? $_GET['imgsrc'] : null;
        $metaInformation['meta']['format'] = isset($_GET['format']) ? $_GET['format'] : 'jpg';
        $metaInformation['meta']['sourceFormat'] = isset($_GET['sourceformat']) ? $_GET['sourceformat'] : 'jpg';

        return $metaInformation;
    }

    public function __call($name, $args)
    {
        if ($name == 'has') {
            $call = $this->getCalledFunctionName($name, $args);
            if (is_callable([$this, $call])) {
                return call_user_func([$this, $call]);
            }
        } elseif(strpos($name, 'getAttribute') !== false) {

            $propertyName = substr($name, strlen('getAttribute'), strlen($name));
            $attributeCall = "get".$propertyName;

            if (method_exists($this, $attributeCall)) {
                return call_user_func([$this, $attributeCall]);
            } else {
                return $this->meta['meta'][strtolower($propertyName)];
            }
        }
    }

    protected function getCalledFunctionName($name, $args) {
        return $name . ucfirst($args[0]);
    }


    /**
     * Return bool if zoom parameter is passed in request
     * @return bool
     */
    public function hasZoom()
    {
        $zoom = (double)$this->meta['meta']['zoom'];
        if (is_double($zoom) || is_int($zoom)) {
            return true;
        }
        return false;
    }

    public function getGeneratedUrlString()
    {
        $c = 0;
        $string = "/".$this->meta['path'] . "/img/";

        if($this->usesExternalImageSource()) {
            $string .= urlencode(str_replace('http:/', 'http://', $this->meta['image'])."/");
        } else {
            $string .= $this->meta['image'] . "/";
        }
        foreach ($this->meta['meta'] as $key => $a) {
            if (null != $a) {
                $string .= $key . "_" . $a . "_";
            }
            $c++;
        }
        return rtrim($string, '_');
    }


    /**
     * Check if image contains a http in that case we are using an external source
     * @return bool
     */
    public function usesExternalImageSource() {
        if(strpos($this->meta['image'], 'http') !== false) {
            return true;
        }
        return false;
    }

    /**
     * Return external image url and add an extra slash if one is missing
     * @return mixed
     */
    public function getExternalImageUrl() {
        return str_replace('http:/', 'http://', $this->meta['image']);
    }


    private function getFormat() {
        if(array_key_exists('format', $this->meta['meta'])) {
            return $this->meta['meta']['format'];
        }
        return 'jpg';
    }
    private function getF() {
        if(array_key_exists('f', $this->meta['meta'])) {
            return $this->meta['meta']['f'];
        }
        return 'cover';
    }

    private function getPercent() {
        if(array_key_exists('percent', $this->meta['meta'])) {
        return $this->meta['meta']['percent'];
    }
        return null;
    }
    private function getQ() {
        if(array_key_exists('q', $this->meta['meta'])) {
            return $this->meta['meta']['q'];
        }
        return 70;
    }

    private function getType() {
        if(array_key_exists('type', $this->meta['meta'])) {
            return $this->meta['meta']['type'];
        }
        return 'preview';
    }

    private function getW() {
        if(array_key_exists('w', $this->meta['meta'])) {
            return $this->meta['meta']['w'];
        }
        return 500;
    }

    private function getH() {
        if(array_key_exists('h', $this->meta['meta'])) {
            return $this->meta['meta']['h'];
        }
        return 'auto';
    }

    private function getX() {
        if(array_key_exists('x', $this->meta['meta'])) {
            return $this->meta['meta']['x'];
        }
        return 0;
    }

    private function getY() {
        if(array_key_exists('y', $this->meta['meta'])) {
            return $this->meta['meta']['y'];
        }
        return 0;
    }

    private function getCrop_w() {
        if(array_key_exists('crop_w', $this->meta['meta'])) {
            return $this->meta['meta']['crop_w'];
        }
        return 0;
    }
    private function getCrop_h() {
        if(array_key_exists('crop_h', $this->meta['meta'])) {
            return $this->meta['meta']['crop_h'];
        }
        return 0;
    }

    private function getZ() {
        if(array_key_exists('z', $this->meta['meta'])) {
            return $this->meta['meta']['z'];
        }
        return null;
    }
}
