<?php
require_once 'includes/Imengine/lib/Image_Adapter.php';
require_once 'includes/Imengine/lib/Storage_Adapter.php';

class Imengine {

	private $_image = null;
	private $_im_adapter = null;
	private $_st_adapter = null;
	private $_image_data = null;
	private $_hash = "";
	private $_quality = null;
    private $_format = 'jpg';
    private $_fileExtension = '.jpg';

	private $_chain = array();

	public function __construct( $image,
                                 Image_Adapter $im_adapter = null,
                                 Storage_Adapter $st_adapter = null,
                                 $image_data = null,
                                 $quality = null,
                                 $format = 'jpg') {

		$this->_image      = $image;
		$this->_im_adapter = $im_adapter;
		$this->_st_adapter = $st_adapter;
		$this->_hash       = md5( $this->_image );
		$this->_image_data = $image_data;
		$this->_quality    = (int)$quality;
        $this->_format = $format;
        $this->setFileExtension();

		return $this;
	}

    /**
     * Send headers for rendered image
     * Currently
     * 200 Ok
     * 404 Not Found
     * @param int $code
     */
    public function set_image_headers($code = 200){

        if($code == 200) {
            $this->set_ok_header();
            $seconds_to_cache = (3600 * 24);
        } elseif($code == 404) {
            $this->set_not_found_header();
            $seconds_to_cache = (1800);
        }

        $this->set_content_type_headers();


        header('Pragma: public');
		header('Last-Modified: '.gmdate('r', time()));
		$ts = gmdate("D, d M Y H:i:s", time() + $seconds_to_cache) . " GMT";
		header("Expires: $ts");
		header("Cache-Control: max-age=$seconds_to_cache");
	}

    /**
     * Send 200 Ok header
     */
    public function set_ok_header() {
        header( "HTTP/1.1 200 OK" );
    }

    /**
     * Send 404 Not Found header
     * Used when an external image is not found.
     */
    public function set_not_found_header() {
        header( "HTTP/1.1 404 Not Found" );
    }
	/*
	 * 	Public Methods
	 */

	public function get_width() {
		return $this->_im_adapter->get_width();
	}

	public function get_height() {
		return $this->_im_adapter->get_height();
	}

	public function thumbnail( $width, $height, $start_x = 0, $start_y = 0 ) {

        $this->beforeRender();

		$file       = $this->_hash . '_thumb_' . (int) $width . 'x' . (int) $height . 'q' . $this->_quality . $this->_fileExtension;
		$image_data = $this->_st_adapter->load( $file );
//        $image_data = null;

		if ( $image_data === null ) {
			if ( $this->_im_adapter->create_image( $this->_image_data, $this->_format ) ) {
				$this->resize_crop( $width, $height );
				$image_data = $this->_im_adapter->generate( $this->_quality );
				$this->_st_adapter->store( $file, $image_data );
			} else {
				// Cant create image
			}
		}
		$this->set_image_headers();
		echo $image_data;

	}

	public function np_crop( $width, $height, $start_x, $start_y, $zoom = 100 ){

        $this->beforeRender();
		$file       = $this->_hash . '_npcrop_' . (int) $width . 'x' . (int) $height . '-' . $start_x .  'x' . $start_y . 'q' . $this->_quality . 'z' . $zoom .  $this->_fileExtension;
		$image_data = $this->_st_adapter->load( $file );
//        $image_data = null;

		if ( $image_data === null ) {
			if ( $this->_im_adapter->create_image( $this->_image_data ) ) {
//                $percent = ( $zoom / 100 );
                $percent = $zoom;

				if ( $zoom !== 100 ) {
                    $this->_im_adapter->crop( $start_x, $start_y, ($this->_im_adapter->get_width())-$start_x, ($this->_im_adapter->get_height())-$start_y );
				}
                $this->_im_adapter->scale( $percent );
                $this->_im_adapter->crop( 0, 0, $width, $height );

				$image_data = $this->_im_adapter->generate( $this->_quality );
				$this->_st_adapter->store( $file, $image_data );
			} else {
				// Cant create image

			}
		}

		$this->set_image_headers();
		echo $image_data;
	}

	public function cover( $width = 'auto', $height = 'auto' ) {

        $this->beforeRender();

		$width  = preg_replace( "/[^a-z0-9]{1,}/i", "", $width );
		$height = preg_replace( "/[^a-z0-9]{1,}/i", "", $height );

		$file       = $this->_hash . '_cover_' . $width . 'x' . $height . 'q' . $this->_quality .  $this->_fileExtension;
		$image_data = $this->_st_adapter->load( $file );
//        $image_data = null;

		if ( $image_data === null ) {
			if ( $this->_im_adapter->create_image( $this->_image_data ) ) {

				if ( $width == 'auto' && $height > 0 ) {
					$width = $this->_im_adapter->get_width() * ( $height / $this->_im_adapter->get_height() );
				} elseif ( $width > 0 && $height == 'auto' ) {
					$height = $this->_im_adapter->get_height() * ( $width / $this->_im_adapter->get_width() );
				} else {
					//echo "Need correct values!";
					//die();
				}
				$this->_im_adapter->resize( $width, $height );
				$image_data = $this->_im_adapter->generate( $this->_quality );


				$this->_st_adapter->store( $file, $image_data );
			}
		}

		$this->set_image_headers();
		echo $image_data;
	}

    public function cropresize( $width, $height, $start_x = 0, $start_y = 0, $crop_w, $crop_h, $z = 100 ) {

        $this->beforeRender();

        $file       = $this->_hash . '_cropresize_' . (int) $width . 'x' . (int) $height . 'x' . (int) $start_x . 'x' . (int) $start_y . 'x' . (int) $crop_w . 'x' . (int) $crop_h . 'x' . (int) $z .'q' . $this->_quality .  $this->_fileExtension;
        $image_data = $this->_st_adapter->load( $file );
//        $image_data = null;

        if ( $image_data === null ) {

            if ( $this->_im_adapter->create_image( $this->_image_data ) ) {
                $this->resize( $width, $height );
                $this->crop( $start_x, $start_y, $crop_w, $crop_h );
                $image_data = $this->_im_adapter->generate( $this->_quality );
                $this->_st_adapter->store( $file, $image_data );
            } else {
                // Cant create image
            }
        }

        $this->set_image_headers();
        echo $image_data;
    }

    public function remake( $width, $height, $start_x = 0, $start_y = 0 ) {

        $this->beforeRender();

        $file       = $this->_hash . '_remake_' . (int) $width . 'x' . (int) $height . 'x' . (int) $start_x . 'x' . (int) $start_y . 'q' . $this->_quality .  $this->_fileExtension;
        $image_data = $this->_st_adapter->load( $file );
//        $image_data = null;

        if ( $image_data === null ) {
            if ( $this->_im_adapter->create_image( $this->_image_data ) ) {
                $this->crop( $start_x, $start_y, $width, $height );
                $image_data = $this->_im_adapter->generate( $this->_quality );
                $this->_st_adapter->store( $file, $image_data );
            } else {
                // Cant create image
            }
        }

        $this->set_image_headers();
        echo $image_data;

    }

    public function cropcover( $width = 'auto', $height = 'auto' , Array $size ) {

        $this->beforeRender();

        $file       = $this->_hash . '_cropcover_' . (int) $width . 'x' . (int) $height . 'x' . (int)preg_replace("/[^0-9]{1,}/i", "", join($size, "")) . 'q' . $this->_quality .  $this->_fileExtension;
        $image_data = $this->_st_adapter->load( $file );
//    	$image_data = null;

        if($image_data === null) {
            if ( $this->_im_adapter->create_image( $this->_image_data ) ) {

                $top = round($this->_im_adapter->get_height()*$size[0]);
                $left = round($this->_im_adapter->get_width()*$size[1]);
                $h = round( ($this->_im_adapter->get_height()) * $size[2] )-$top;
                $w = round( ($this->_im_adapter->get_width()) * $size[3] )-$left;

                $this->crop($left, $top, $w, $h);

                if ( $width == 'auto' && $height > 0 ) {
                    $width = $this->_im_adapter->get_width() * ( $height / $this->_im_adapter->get_height() );
                } elseif ( $width > 0 && $height == 'auto' ) {
                    $height = $this->_im_adapter->get_height() * ( $width / $this->_im_adapter->get_width() );
                } else {
                    // No correct values
                }

                $this->_im_adapter->resize( $width, $height );
            	
                $image_data = $this->_im_adapter->generate( $this->_quality );
            } else {
                // Cant create image
            }
        }

		$this->set_image_headers();
        echo $image_data;
    }

	/*
	 * Image methods
	 */

	public function load() {
		$this->_im_adapter->create_image( $this->_image_data );
	}

	public function resize_crop( $width, $height ) {
		$this->_im_adapter->resize( $width, $height, 'maximal' );

		$source_ratio = $this->_im_adapter->get_width() / $this->_im_adapter->get_height();
		$thumb_ratio  = $width / $height;

		if ( $thumb_ratio < $source_ratio ) {
			$start_x = ( $this->_im_adapter->get_width() - $width ) / 2;
			$this->_im_adapter->crop( $start_x, 0, $width, $height );
		} else {
			$start_y = ( $this->_im_adapter->get_height() - $height ) / 2;
			$this->_im_adapter->crop( 0, $start_y, $width, $height );
		}

		return $this;
	}

	public function crop( $start_x, $start_y, $width, $height ) {
		$this->_im_adapter->crop( $start_x, $start_y, $width, $height );
		return $this;
	}

    public function percent_crop( Array $size ) {

    }

	public function resize( $width, $height ) {
		$this->_im_adapter->resize( $width, $height );
		return $this;
	}

	public function display( $quality = 90 ) {
		$this->set_image_headers();
		echo $this->_im_adapter->generate( $quality );
	}

    private function beforeRender()
    {
        if($this->_image_data == '404') {
            $this->set_image_headers(404);
        } elseif($this->_image_data == null) {
            $this->set_image_headers(404);
        }
    }

    /**
     * Check property format to check which file extension to use
     * @return string
     */
    private function setFileExtension()
    {
        if($this->_format == 'png') {
            $this->_fileExtension = '.png';
        } elseif($this->_format == 'gif') {
            $this->_fileExtension = '.gif';
        }
        $this->_fileExtension = '.jpg';
    }

    /**
     * Send correct content type for image format
     */
    protected function set_content_type_headers()
    {
        if ($this->_format == 'jpg') {
            header("Content-type: image/jpeg");
        } elseif ($this->_format == 'png') {
            header("Content-type: image/png");
        } elseif ($this->_format == 'gif') {
            header("Content-type: image/gif");
        }
    }

    /*
     * Storage methods
     */

}