<?php
	
	class GD_Adapter extends Image_Adapter {
		
		private $_im;
		private $_current;
		private $_width;
		private $_height;
		private $_format;

		public function create_image($image, $format = 'jpg') {
			$this->_im = imagecreatefromstring($image);

            $this->_format = $format;

			if(phpversion() >= 5.4) {
				list($this->_width, $this->_height, $source_type) = getimagesizefromstring($image);
			} else {
				$this->_width = imagesx($this->_im);
				$this->_height = imagesy($this->_im);
			}
			
			/*
			switch ($source_type) {
		        case IMAGETYPE_GIF:
		            $this->_im = imagecreatefromgif($image);
		            break;
		        case IMAGETYPE_JPEG:
		            $this->_im = imagecreatefromjpeg($image);
		            break;
		        case IMAGETYPE_PNG:
		            $this->_im = imagecreatefrompng($image);
		            break;
				default:
					$this->_im = imagecreatefromstring(file_get_contents($image));
					break;
		    }
			*/
			if ($this->_im === false) {
				echo _e("No image type supported", "every");
		    	return false;
		    }
			
			return true;
		}
		
		public function get_width() { return $this->_width; }
		public function get_height() { return $this->_height; }
		
		public function crop($start_x, $start_y, $width = 'auto', $height = 'auto') {
			
			
			$width = $width == 'auto' ? $this->_width-$start_x : $width;
			$height = $height == 'auto' ? $this->_height-$start_y : $height;
			
			$this->_current = imagecreatetruecolor($width, $height);

			#imagecopyresampled($this->_current, $this->_im, 0, 0, $start_x, $start_y, $this->_width, $this->_height, $this->_width, $this->_height);
            imagecopyresampled($this->_current, $this->_im, 0, 0, $start_x, $start_y, $this->_width, $this->_height, $this->_width, $this->_height);
			
			$this->_im = $this->_current;
			$this->_width = $width;
			$this->_height = $height;
			
		}
				
		public function resize($width, $height, $type = 'minimal') {
			$new_max_width = $thumb_width = $width;
			$new_max_height = $thumb_height = $height;
			
			$source_ratio = $this->_width / $this->_height;
			$thumb_ratio = $new_max_width / $new_max_height;
			
			if( $this->_width <= $new_max_width && $this->_height <= $new_max_height ) {
				$thumb_width = $this->_width;
				$thumb_height = $this->_height;
			} else {
				
				if($type == 'minimal') {
					if( $thumb_ratio > $source_ratio ) {
						$thumb_width = (int)($source_ratio*$new_max_height);
						$thumb_height = $new_max_height;
					} else {
						$thumb_width = $new_max_width;
						$thumb_height = (int)(($this->_height / $this->_width)*$new_max_width);
					}
				} else {
					if( $thumb_ratio < $source_ratio ) {
						$thumb_width = (int)($source_ratio*$new_max_height);
						$thumb_height = $new_max_height;
					} else {
						$thumb_width = $new_max_width;
						$thumb_height = (int)(($this->_height / $this->_width)*$new_max_width);
					}
				}
			
			}
			
			$this->_current = imagecreatetruecolor($thumb_width, $thumb_height);
			imagecopyresampled($this->_current, $this->_im, 0, 0, 0, 0, $thumb_width, $thumb_height, $this->_width, $this->_height);
			
			$this->_width = $thumb_width;
			$this->_height = $thumb_height;
			$this->_im = $this->_current;
		}

		public function scale( $percent ){
			$new_width  = $this->get_width() * $percent;
			$new_height = $this->get_height() * $percent;

			//$this->_im->scaleImage( $new_width, $new_height );
            //$this->_current = imagecreatetruecolor($this->_width, $this->_height);
            $this->_current = imagecreatetruecolor($new_width, $new_height);
            imagecopyresampled($this->_current, $this->_im, 0, 0, 0, 0, $new_width, $new_height, $this->_width, $this->_height);
            $this->_im = $this->_current;
		}

		public function generate($quality) {
			
			ob_start();
			//imagefilter($this->_current, IMG_FILTER_GRAYSCALE);
            if($this->_format == 'png') {
                $quality = $quality/10;
                imagepng($this->_current, null, $quality);
            } elseif($this->_format == 'jpg') {
                imagejpeg($this->_current, null, $quality);
            } elseif($this->_format == 'gif') {
                imagegif($this->_current, null);
            }

			$content = ob_get_contents();
			ob_end_clean();
			
			return $content;
		}
		
	}
