<?php

/*
 * This class is responsible for creating a custom post type of an Article object
 * Code moved here to prevent unwanted use of this function directly from an OcArticle object
 * This function is being used from OcConnection object, get_single_article and create_article_object
 */

class OcArticleCustomPostFactory {

	CONST WEB_HEADLINE = 'web_headline';
	CONST TABLET_HEADLINE = 'tablet_headline';
	CONST MOBILE_HEADLINE = 'mobile_headline';

	private $article;

	public function __construct( OcArticle $article ) {
		$this->article = $article;
		$this->create_oc_article_posttype();
	}

	/*
	 * Public function to generate an custom post type object of each article
	 * Previously done by __construct but since dynamical props where implemented
	 * this function must be called after instantiation of an OcArticle object to
	 * guarantee that all props have been set
	 *
	 * @scope - public
	 * @param -
	 * @return -
	 */
	public function create_oc_article_posttype() {
        $trans_key = 'pe_' . $this->article->uuid[0];

            if( 'true' === get_transient($trans_key) ) {
            return;
        }

		/*$args = array(
			'post_type'  => 'article',
			'posts_per_page' => 1,
			'meta_query' => array(
				array(
					'key'     => 'oc_uuid',
					'value'   => $this->article->uuid[0],
					'compare' => '='
				)
		));*/

        #$article_arr = get_posts( $args );
        set_transient($trans_key, 'true', 3600);
        $article_check = OcUtilities::get_article_post_id_by_uuid($this->article->uuid[0]);

		//if ( is_array( $article_arr ) && count( $article_arr ) === 0 ) {
        if($article_check == null) {
			// If tags is an array convert it to a comma separated string
			$wp_tags = $this->article->tags;
			if ( is_array( $this->article->tags ) ) {
				$wp_tags = implode( ",", $this->article->tags );
			}

            $env_settings   = new EnvSettings();
            $cat_prop       = $env_settings->get_category_property();
	        $hide_future    = $env_settings->get_pubdate_hide_property();

            $cat_arr        = isset( $this->article->$cat_prop ) ? $this->article->$cat_prop : '';
	        $category       = isset( $cat_arr[0] ) ? urldecode( sanitize_title_with_dashes( $cat_arr[0] ) ) : '';

			$post_date 		= isset( $this->article->created[0] ) ? $this->article->created[0] : date('Y-m-d');
			$post_time		= strtotime( $post_date );
			$post_date		= date( 'Y-m-d H:i:s', $post_time );
			$post_date_gmt 	= gmdate( 'Y-m-d H:i:s', $post_time );

	        $post_status = 'publish';
	        if( $hide_future && isset( $this->article->pubdate[0] ) ) { // If articles with pubdates in the future should be hidden.
		        $pubdate_timestamp  = strtotime( $this->article->pubdate[0] );
		        $now_timestamp      = time();

		        if( $pubdate_timestamp > $now_timestamp ) {
			        $post_status    = 'future';
			        $post_date		= date( 'Y-m-d H:i:s', $pubdate_timestamp );
			        $post_date_gmt 	= gmdate( 'Y-m-d H:i:s', $pubdate_timestamp );
			    }
	        }

			// Set data for custom post type Article
			$article                 	= array();
			$article['post_type']    	= 'article';
			$article['post_content'] 	= '';
			$article['post_parent']  	= 0;
			$article['post_status']  	= $post_status;
			$article['post_title']   	= $this->generate_oc_article_slug();
			$article['post_name']    	= $article['post_title'];
			$article['tags_input']   	= $wp_tags;
			$article['post_date']		= $post_date;
			$article['post_date_gmt']	= $post_date_gmt;

			#$web_headline = isset($this->article->web_headline[0]) ? $this->article->web_headline[0] : $article['post_title'];
			#$tablet_headline = isset($this->article->tablet_headline[0]) ? $this->article->tablet_headline[0] : $article['post_title'];
			#$mobile_headline = isset($this->article->mobile_headline[0]) ? $this->article->mobile_headline[0] : $article['post_title'];

			if ( $category !== '' ) {
				$category_id = get_cat_id( $category );
				if ( $category_id == 0 ) {
					$category_id = wp_insert_term( $category, 'category' );
				}
				$article['post_category'] = array( $category_id );
			}
			else {
				$article['post_category'] = array( 1 );
			}


	        // Create the page and set it's template
			$post_id = wp_insert_post( $article );


            //TODO: Tell OC that the article has been added
						
			if ( $post_id !== 0 ) {
                set_transient($trans_key, 'true', 3600);
				$post_id = absint( $post_id );
				update_post_meta( $post_id, '_wp_page_template', 'single-article.php' );
				add_post_meta( $post_id, 'oc_uuid', $this->article->uuid[0], $unique = true );
				add_post_meta( $post_id, '_customize_sidebars', 'yes' );
				#add_post_meta( $post_id, self::WEB_HEADLINE, $web_headline, $unique = true );
				#add_post_meta( $post_id, self::TABLET_HEADLINE, $tablet_headline, $unique = true );
				#add_post_meta( $post_id, self::MOBILE_HEADLINE, $mobile_headline, $unique = true );
				add_post_meta( $post_id, EveryStats::EVERY_STATS_COUNT_KEY, 0, true );

                do_action( 'every_article_created', $post_id, $this->article->uuid[0] );
			} else {
                set_transient($trans_key, 'false', 3600);
            }
		} else {
            set_transient($trans_key, 'true', 3600);
        }
	}

	/*
	 * Private function that generates a slug for the given article object.
	 * @scope - private
	 * @param OcArticle $article
	 * @return string
	 */
	private function generate_oc_article_slug() {
//		$oc              = new OpenContent();
		$oc              = OpenContent::getInstance();
		$slug_properties = $oc->getSlugProperties();
		$slug_max_length = $oc->getSlugMaxLength();
		$slug            = '';

		if ( isset( $slug_properties ) ) {
			foreach ( $slug_properties as $property ) {
				$property = strtolower( $property );
				if ( isset( $this->article->$property ) && $this->article->$property !== '' ) {
					$value = $this->article->$property;
					if ( isset( $value[0] ) && $value[0] !== '' ) {
						$slug = mb_substr( strip_tags( $value[0] ), 0, $slug_max_length );
						break;
					}
				}
			}
		}

		return $slug;
	}
}