<?php


class Action
{

    public $data = [];
    public $time;
    protected static $instance;
    protected $perfunction = [];
    public $addedToDb;
    public $requestStart;

    public $analyzer_disabled = false;

    /**
     * Get a singleton instance of the class
     *
     * @return self
     * @codeCoverageIgnore
     */
    public static function getInstance()
    {
        if (!self::$instance) {
            self::$instance = new self();
        }
        return self::$instance;
    }

    public function addPerfunction($function, $item) {

        $this->perfunction[$function][] = $item;
    }

    public function add($data, $type)
    {

        if($this->analyzer_disabled) {
            return;
        }
        $key = uniqid();
        $this->data[$key]['data'] = $data;

        $this->data[$key]['time'] = microtime(false);
        $this->addPerfunction($this->data[$key]['data']['debug'][0]['function'], $this->data[$key]['data']['debug'][0]);

        $name = $data['name'];
        $function =  $this->data[$key]['data']['debug'][1]['function'];
        $file = $this->data[$key]['data']['debug'][1]['file'];
        $args = $this->data[$key]['data']['debug'][1]['args'][0] ;
        $loadtime = $this->data[$key]['data']['debug'][1]['time'][0];

        $functionsCalled = $this->getCalledFunctions( $this->data[$key]['data']['debug'] );
        $dataToSave = [
            'functions' => $functionsCalled,
            'data'      => $data['debug']
        ];

        global $wpdb, $ocanalyzer_request_id;
        $table_name = $wpdb->prefix . 'ocanalyzer';

        $wpdb->insert(
            $table_name,
            array(
                'time'          => current_time( 'mysql' ),
                'name'          => $name,
                'function'      => $function,
                'file'          => $file,
                'argument'      => $args,
                'data'          => serialize($dataToSave),
                'type'          => $type,
                'ocanalyzer_id' => $ocanalyzer_request_id,
                'start_request' => round($_SERVER["REQUEST_TIME_FLOAT"],1),
                'loadtime'      => $loadtime,
                'request_uri'       => $_SERVER['REQUEST_URI']
            )
        );

        $this->addedToDb++;

    }

    public function getCalledFunctions($data) {

        $functionsCalled = [];
        foreach ($data as $d) {
            $functionsCalled[] = $d['function'];
        }
    return $functionsCalled;

    }

    public function render()
    {
        if($this->analyzer_disabled) {
            return;
        }
        if(isset($_GET['hide'])) {
            return;
        }

        $start = $_SERVER["REQUEST_TIME_FLOAT"];
        $this->requestStart = $start;

        echo "<script>";
        echo "jQuery.ajax({
                type : 'get',
                dataType : 'html',
                url: '".admin_url( 'admin-ajax.php')."',
                data : {
                    action: 'load_ocanalyzer_db',
                    request: ".$start."
                },
                success: function(response) {
                    jQuery('#ocanalyzer_database').html(response);
                },
                error: function(qXHR, textStatus, errorThrow) {}
            });";

        echo "


        ";
        echo "</script>";
        echo "<style>";
        echo "
            body {padding-bottom:500px;}
            .ocanalyzer_data {
                border:1px solid #ccc;
            }
            .ocanalyzer_data tr {
                border:1px solid #ccc;
            }

        ";
        echo "</style>";

        echo "<div id='ocanalyzer_container' style='position:fixed; left:0; bottom:0; height:400px; width: 100%; background-color: #fff; border-top:1px solid #ccc; box-shadow: 0 -2px 2px #ccc; z-index: 9999; overflow: scroll;padding:20px; color: #666; font-size:14px'>";

        // ========== Request =============
        echo "<div style='width:49%; float:left;'>";
            echo "<table class='ocanalyzer_data' border='0' style='width:100%; border:1px solid #ccc'>";

            foreach ($this->perfunction as $key =>  $function) {

                echo "<tr>";
                echo "<td style='padding:3px;' colspan='3'>";
                echo "<h4>".$key ." called ".count($function)." times</h4>";
                echo "</td>";
                echo "<tr>";

                $count = 0;
                foreach($function as $item) {
                    $time = microtime(true) - $start;
                    if($count % 2 == 0) :
                        echo "<tr style='background-color: #efefef'>";
                    else:
                        echo "<tr style='background-color: #fff'>";
                    endif;
                    echo "<td style='padding:3px;'>".$this->shortenFileName($item['file'])."</td>";
                    echo "<td style='padding:3px;'><a href='".$item['args'][0]."'>".$this->shortenUrl($item['args'][0])."</a></td>";
                    echo "<td style='padding:3px;'>After request " . $time . "</td>";
                    echo "<tr>";
                    $count++;
                }
            }
            echo "</table>";
        echo "</div>";



        // ========== DB =============
        echo "<div style='width:49%; float:right;' id='ocanalyzer_database'>";
            echo "Loading results from database...";
        echo "</div>";



        echo "</div>";
    }


    public function renderDatabase($request) {

        if($this->analyzer_disabled) {
            return;
        }
        sleep(4); // Sleep to let db makes insertions from CLI and Imengine
        global $wpdb;

        $rows = $wpdb->get_results("SELECT * FROM ".$wpdb->prefix."ocanalyzer WHERE start_request >= ".round($request,1));

        echo "<table  class='ocanalyzer_data' border='1' style='width:100%'>";

        echo "<tr>";
        echo "<td style='padding:3px;' colspan='4'>";
        echo "<h4>Database results</h4>";
        echo "</td>";
        echo "<tr>";

        $count = 0;
        foreach ($rows as $key =>  $row) {
            if($count % 2 == 0) :
                echo "<tr style='background-color: #efefef'>";
            else:
                echo "<tr style='background-color: #fff'>";
            endif;
            echo "<td style='padding:3px;'>".$row->type."</td>";
            echo "<td style='padding:3px;'>".$this->shortenFileName($row->file)."</td>";
            echo "<td style='padding:3px;'><a href='".$row->argument."'>".$this->shortenUrl($row->argument)."</a></td>";
            echo "<td style='padding:3px;'>Loadtime " . $row->loadtime . "</td>";
            echo "<tr>";

            $count++;
        }
        echo "</table>";

        echo "<div style='width: 100%'>";
        echo "<strong>Request resulted in ".count($rows)." records where added to db</strong>";
        echo "</div>";

    }


    public function shortenFileName($file) {
        $parts = preg_split('/\//', $file);
        $new = array_slice($parts, count($parts)-3, 3);
        return implode('/', $new);
    }

    public function shortenUrl($url) {
        $parts = parse_url($url);
        return $parts['path'];
    }

}

